@@ -97,43 +97,43 @@
 
 G_BEGIN_DECLS
 
-GST_EXPORT
+GST_API
 void		gst_init			(int *argc, char **argv[]);
 
-GST_EXPORT
+GST_API
 gboolean	gst_init_check			(int *argc, char **argv[],
-						 GError ** err);
-GST_EXPORT
+						 GError ** error);
+GST_API
 gboolean        gst_is_initialized              (void);
 
-GST_EXPORT
+GST_API
 GOptionGroup *	gst_init_get_option_group	(void);
 
-GST_EXPORT
+GST_API
 void		gst_deinit			(void);
 
-GST_EXPORT
+GST_API
 void		gst_version			(guint *major, guint *minor,
 						 guint *micro, guint *nano);
-GST_EXPORT
+GST_API
 gchar *		gst_version_string		(void);
 
-GST_EXPORT
+GST_API
 gboolean        gst_segtrap_is_enabled          (void);
 
-GST_EXPORT
+GST_API
 void            gst_segtrap_set_enabled         (gboolean enabled);
 
-GST_EXPORT
+GST_API
 gboolean        gst_registry_fork_is_enabled    (void);
 
-GST_EXPORT
+GST_API
 void            gst_registry_fork_set_enabled   (gboolean enabled);
 
-GST_EXPORT
+GST_API
 gboolean        gst_update_registry             (void);
 
-GST_EXPORT
+GST_API
 const gchar *   gst_get_main_executable_path    (void);
 
 G_END_DECLS
