@@ -14,17 +14,17 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ * MA 02110-1301, USA.
  */
 
-#include "config.h"
+#include "libcompat/libcompat.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "check.h"
+#include "internal-check.h"
 #include "check_list.h"
 #include "check_impl.h"
 #include "check_str.h"
@@ -83,14 +83,63 @@ srunner_fprint_results (FILE * file, SRunner * sr, enum print_output print_mode)
 
   resultlst = sr->resultlst;
 
-  for (list_front (resultlst); !list_at_end (resultlst);
-      list_advance (resultlst)) {
-    TestResult *tr = list_val (resultlst);
+  for (check_list_front (resultlst); !check_list_at_end (resultlst);
+      check_list_advance (resultlst)) {
+    TestResult *tr = (TestResult *) check_list_val (resultlst);
+
     tr_fprint (file, tr, print_mode);
   }
   return;
 }
 
+void
+fprint_xml_esc (FILE * file, const char *str)
+{
+  /* The valid XML characters are as follows:
+   *   #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]
+   * Characters that are outside of ASCII must be encoded. Further, the
+   * following special characters:
+   *   " ' < > &
+   * must be encoded. We assume that the incoming string may be a multibyte
+   * character.
+   */
+
+  for (; *str != '\0'; str++) {
+    char next = *str;
+
+    /* handle special characters that must be escaped */
+    if (next == '"' || next == '\'' || next == '<' || next == '>'
+        || next == '&') {
+      switch (next) {
+        case '"':
+          fputs ("&quot;", file);
+          break;
+        case '\'':
+          fputs ("&apos;", file);
+          break;
+        case '<':
+          fputs ("&lt;", file);
+          break;
+        case '>':
+          fputs ("&gt;", file);
+          break;
+        case '&':
+          fputs ("&amp;", file);
+          break;
+      }
+    }
+    /* printable ASCII */
+    else if (next >= ' ' && next <= '~') {
+      fputc (next, file);
+    }
+    /* Non-printable character */
+    else if (next == 0x9 || next == 0xA || next == 0xD || next >= 0x20) {
+      fprintf (file, "&#x%X;", next);
+    }
+    /* If it did not get printed, it is not a valid XML character */
+  }
+}
+
 void
 tr_fprint (FILE * file, TestResult * tr, enum print_output print_mode)
 {
@@ -101,61 +150,30 @@ tr_fprint (FILE * file, TestResult * tr, enum print_output print_mode)
   if ((print_mode >= CK_VERBOSE && tr->rtype == CK_PASS) ||
       (tr->rtype != CK_PASS && print_mode >= CK_NORMAL)) {
     char *trstr = tr_str (tr);
+
     fprintf (file, "%s\n", trstr);
     free (trstr);
   }
 }
 
-static void
-fprint_xml_esc (FILE * file, const char *str)
-{
-  for (; *str != '\0'; str++) {
-
-    switch (*str) {
-
-        /* handle special characters that must be escaped */
-      case '"':
-        fputs ("&quot;", file);
-        break;
-      case '\'':
-        fputs ("&apos;", file);
-        break;
-      case '<':
-        fputs ("&lt;", file);
-        break;
-      case '>':
-        fputs ("&gt;", file);
-        break;
-      case '&':
-        fputs ("&amp;", file);
-        break;
-
-        /* regular characters, print as is */
-      default:
-        fputc (*str, file);
-        break;
-    }
-  }
-}
-
 void
 tr_xmlprint (FILE * file, TestResult * tr,
     enum print_output print_mode CK_ATTRIBUTE_UNUSED)
 {
   char result[10];
-  char *path_name;
-  char *file_name;
-  char *slash;
+  char *path_name = NULL;
+  char *file_name = NULL;
+  char *slash = NULL;
 
   switch (tr->rtype) {
     case CK_PASS:
-      strcpy (result, "success");
+      snprintf (result, sizeof (result), "%s", "success");
       break;
     case CK_FAILURE:
-      strcpy (result, "failure");
+      snprintf (result, sizeof (result), "%s", "failure");
       break;
     case CK_ERROR:
-      strcpy (result, "error");
+      snprintf (result, sizeof (result), "%s", "error");
       break;
     case CK_TEST_RESULT_INVALID:
     default:
@@ -163,22 +181,33 @@ tr_xmlprint (FILE * file, TestResult * tr,
       break;
   }
 
-  slash = strrchr (tr->file, '/');
-  if (slash == NULL) {
-    path_name = (char *) ".";
-    file_name = tr->file;
-  } else {
-    path_name = strdup (tr->file);
-    path_name[slash - tr->file] = 0;    /* Terminate the temporary string. */
-    file_name = slash + 1;
+  if (tr->file) {
+    slash = strrchr (tr->file, '/');
+    if (slash == NULL) {
+      slash = strrchr (tr->file, '\\');
+    }
+
+    if (slash == NULL) {
+      path_name = strdup (".");
+      file_name = tr->file;
+    } else {
+      path_name = strdup (tr->file);
+      path_name[slash - tr->file] = 0;  /* Terminate the temporary string. */
+      file_name = slash + 1;
+    }
   }
 
 
   fprintf (file, "    <test result=\"%s\">\n", result);
-  fprintf (file, "      <path>%s</path>\n", path_name);
-  fprintf (file, "      <fn>%s:%d</fn>\n", file_name, tr->line);
+  fprintf (file, "      <path>%s</path>\n",
+      (path_name == NULL ? "" : path_name));
+  fprintf (file, "      <fn>%s:%d</fn>\n",
+      (file_name == NULL ? "" : file_name), tr->line);
   fprintf (file, "      <id>%s</id>\n", tr->tname);
   fprintf (file, "      <iteration>%d</iteration>\n", tr->iter);
+  fprintf (file, "      <duration>%d.%06d</duration>\n",
+      tr->duration < 0 ? -1 : tr->duration / US_PER_SEC,
+      tr->duration < 0 ? 0 : tr->duration % US_PER_SEC);
   fprintf (file, "      <description>");
   fprint_xml_esc (file, tr->tcname);
   fprintf (file, "</description>\n");
@@ -187,15 +216,14 @@ tr_xmlprint (FILE * file, TestResult * tr,
   fprintf (file, "</message>\n");
   fprintf (file, "    </test>\n");
 
-  if (slash != NULL) {
-    free (path_name);
-  }
+  free (path_name);
 }
 
 enum print_output
 get_env_printmode (void)
 {
   char *env = getenv ("CK_VERBOSITY");
+
   if (env == NULL)
     return CK_NORMAL;
   if (strcmp (env, "silent") == 0)
