@@ -19,6 +19,7 @@
 
 /**
  * SECTION:gstnetcontrolmessagemeta
+ * @title: GstNetControlMessageMeta
  * @short_description: Network Control Message Meta
  *
  * #GstNetControlMessageMeta can be used to store control messages (ancillary
@@ -27,6 +28,9 @@
  * sending and receiving ancillary data such as unix credentials (See
  * #GUnixCredentialsMessage) and Unix file descriptions (See #GUnixFDMessage).
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <string.h>
 
@@ -71,7 +75,7 @@ net_control_message_meta_free (GstMeta * meta, GstBuffer * buffer)
 GType
 gst_net_control_message_meta_api_get_type (void)
 {
-  static volatile GType type;
+  static GType type;
   static const gchar *tags[] = { "origin", NULL };
 
   if (g_once_init_enter (&type)) {
